#!/usr/local/bin/awk -E

# "Auto" if you plan to automatically inject the template into the article via AWB F10 mode

# Demon to pass data back and forth between Windows host and Linux guest OS under VirtualBox
# See also demon-win.awk for the version running on the Windows host.
# Purposes is to run AWB on windows, and unix scripts on Linux. 

@include "init.awk"
@include "library.awk"
#@include "getopt.awk"

BEGIN {

  Optind = Opterr = 1
  while ((C = getopt(ARGC, ARGV, "d:p:")) != -1) {
    if(C == "p")                                     # -p <project>      Project to run 
      pid = verifypid(Optarg)
    if(C == "d")                                     # -d <number>       Delay seconds to use (must be > 12). If set, no random delay.
      delay = Optarg                                 #                     If -d not set, a random delay period is used (15 + 1-50 seconds)            
  }
  setProject(pid)     # library.awk .. load Project[] paths via project.cfg
                      # if -p not given, use default noted in project.cfg

  if( delay == "" || delay < 12 || ! isanumber(delay) ) {
    delay = 15 
    delayrandom = 1
  }
  else {
    delay = 15
    delayrandom = 0
  }

  coredelay = 60      # 180 will do ~300 in ~ 9 hours

  main()

}

function main(    name,tempid,article) {

  removefile(Ramdisk "name.txt")              # Clear out old files
  removefile(Ramdisk "article.txt")           
  removefile(Ramdisk "abort.txt")              
  removefile(Ramdisk "done.txt")

  prnt("\ndemon-lin.awk: Waiting for " Ramdisk "name.txt ...")

  while(1) {

    sleep(2)
    Z++
    if(Z > 500) {   # time-out. 2500 = about 90 minutes w/ 2sec sleep
      print "demon-lin.awk: Time out"
      exit
    }

    if( checkexists(Ramdisk "name.txt") ) {

      Z = 0                              # reset time-out
      name=article=tempid=choose=""      # reset variables

      name = strip( readfile(Ramdisk "name.txt") )
      prnt("demon-lin.awk: New job request: " name )
      removefile(Ramdisk "name.txt")
      removefile(Ramdisk "done.txt")

      if(length(name) > 0) {           
        if( nameisinfile(name, Project["manual"] ) ) {
          abort("demon-lin.awk: Status skipping manual. " name)
        }
        else {
          tempid = whatistempid(name, Project["index"] )
          if(tempid == "" || tempid == 0) {
            abort("demon-lin.awk: Error unknown tempid. " name)
          }         
          else {
            if( sys2var( Exe["awk"] " -v id=\"" tempid "\" -f " Home "catignore.awk") == 1 ) {
              abort("demon-lin.awk: Cat ignore. " name)
              sendto(Project["catignore"], name, tempid)
            }
            else {
              # https://en.wikipedia.org/w/index.php?title=Władysław_T._Benda&action=raw
              # article = http2var("https://en.wikipedia.org/w/index.php?title=" gensub(" ","_","g",name) "&action=raw")
              article = getwikisource(name, "dontfollow")
              if(length(article) == 0) {
                abort("demon-lin.awk: Error unable to retrieve wikipedia wikisource. " name)
              }
              else {                                                     # Refresh article.txt 

                sys2var( Exe["mv"] " " tempid "article.txt " tempid "article.txt.demon-lin")
                print article > tempid "article.txt"
                close(tempid "article.txt")

                if(! checkexists(tempid "choose.txt") ) {
                  abort("demon-lin.awk: Error choose.txt not found. " name)
                }
                else {                                                   # Get template from choose.txt

                  j = i = c = 0
                  delete Found
                  choose = readfile(tempid "choose.txt")
                  c = split(choose, a, "\n")
                  while(i++ < c) {
                    safe = strip(a[i])
                    if(safe ~ /Internet Archive author/) {
                      if(substr(safe,length(safe) - 2, length(safe)) ~ / [*]$/) {
                        j++
                        match(safe, /{{Internet Archive author[^}]*}}/, k)
                        Found[j] = k[0]
                      }
                    }
                  }
                  if(j != 1) {
                    sendto(Project["manual"], name, "choosetxt")
                    abort("demon-lin.awk: Error choose.txt has < or > 1 recommendations. Sent to manual. " name)
                  }
                  if(j == 1) {                                          # Inject template into article
                    print "* " Found[1] > tempid "out.ia"
                    close(tempid "out.ia")
                    if( sys2var( Exe["awk"] " -v id=" tempid " -f " Home "extlnks.awk" ) ~ "no" ) {
                      sendto(Project["manual"], name, "insertion")
                      abort("demon-lin.awk: Error unable to insert template into article extlnks.awk. Sent to manual. " name)              
                    } 
                    else {                                              # Copy to shared directory

                      if(checkexists(tempid "article.extlnks.txt") ) {
                        if(length(article) < length(readfile(tempid "article.extlnks.txt"))) {
                          aef = readfile(tempid "article.extlnks.txt")
                          print aef > Ramdisk "article.txt"
                          close(Ramdisk "article.txt")
                          prnt("demon-lin.awk: Status successful. Copied article.extlnks.txt to shared directory. " name) 
                          if(delayrandom)
                            sleep( delay + randomnumber(coredelay) )      
                          else
                            sleep( delay )
                          print "done" > Ramdisk "done.txt"
                          close(Ramdisk "done.txt")
                        }
                        else {  
                          sendto(Project["manual"], name, "articletxtlength")
                          abort("demon-lin.awk: Error article.extlnks.txt length mismatch. Sent to manual. " name)
                        }
                      }
                      else {
                        sendto(Project["manual"], name, "articletxtmissing")
                        abort("demon-lin.awk: Error article.extlnks.txt missing. Sent to manual. " name)
                      }
                    }
                  }
                }
              }
            }
          }
        }
      } 
      else {
        abort("demon-lin.awk: error retrieving name")
      }  
      prnt("\ndemon-lin.awk: Waiting for " Ramdisk "name.txt ...")
    }
  }
}

#
# Return the path/tempid of a name (eg. /home/adminuser/wi-awb/temp/wi-awb-0202173111/)
#
function whatistempid(name, filepath,      s, a, re) {

  if(! checkexists(filepath) ) {
    abort("demon-lin.awk: Error unable to find " filepath ". " name )
    return 0
  }
  re = "^" regesc(strip(name)) "$"
  while ((getline s < filepath ) > 0) {
    split(s, a, "|")
    if(strip(a[1]) ~ re) {
      close(filepath)
      return strip(a[2])
    }
  }
  close(filepath)
  return 0
}

#
# Return 1 if name is in file 
#
function nameisinfile(name, filepath,     s, a, re) {

  if(! checkexists(filepath) ) {
    abort("demon-lin.awk: Error unable to find " filepath ". " name )
    return 0
  }
  re = "^" regesc(strip(name)) "$"
  while ((getline s < filepath ) > 0) {
    split(s, a, "|")
    if(strip(a[1]) ~ re) {
      close(filepath)
      return 1
    }
  }
  close(filepath)
  return 0
}

#
# Abort 
#
function abort(msg,  filen) {
  if( length(msg) > 0 ) 
    prnt(msg)
  printf "0" > Ramdisk "abort.txt"
  close(Ramdisk "abort.txt")
}

#
# Print and log messages
#
function prnt(msg) {
  if( length(msg) > 0 ) {
    print(msg)
    print(strftime("%Y%m%d %H:%M:%S") " " msg) >> Home "demon-lin.log"
    close(Home "demon-lin.log")
  }
}

function removefile(str) {
      if( checkexists(str) )
        sys2var( Exe["rm"] " -- " str)
      if( checkexists(str) ) {
        abort("demon-lin.awk: Error unable to delete " str ", aborting.")
        exit
      }
      system("") # Flush buffer
}

#
# Sleep 
#
function sleep(seconds)
{
  sys2var( Exe["sleep"] " " seconds)
}


